unit EnhTreeView;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls,CommCtrl;

type

  TEnhTreeNode = class (comCtrls.TTreeNode)
  private
    fChecked : Boolean;
    function GetChecked :Boolean;
    procedure SetChecked(value:Boolean);
  public
    constructor CreateEnh(AOwner: TTreeNodes);
    destructor Destroy; override;
    property Checked:Boolean read GetChecked write SetChecked;
  end;

  TEnhTreeView = class(TTreeView)
  private
    { Private declarations }
    fCheckBoxes : Boolean;
    function GetCheckBoxes:Boolean;
    procedure SetCheckBoxes(value:Boolean);
  protected
    { Protected declarations }
    function CreateNode: TTreeNode; virtual;
  public
    { Public declarations }
    procedure CreateParams(var Params: TCreateParams);override;
    procedure WMNOTIFY(var msg : TWMNotify);message CN_NOTIFY;
  published
    { Published declarations }
    property CheckBoxes : boolean read GetCheckBoxes write SetCheckBoxes;
  end;

procedure Register;

implementation

procedure TEnhTreeview.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  if fCheckBoxes then with Params do
    Style := Style or TVS_CHECKBOXES;
end;

procedure TEnhTreeview.WMNOTIFY(var msg : TWMNotify);
var
  Node : TTreeNode;
  Point : TPoint;
begin
  inherited;
  point:=Mouse.CursorPos;
  point:=ScreenToClient(point);
  with Msg,point do
    case NMHdr^.code of
      NM_CLICK,NM_RCLICK : begin
                             Node:=GetNodeAt(x,y);
                             if assigned(Node) then
                               Selected:=Node
                             else begin
                               if fCheckBoxes then begin
                                 Node:=GetNodeAt(x+16,y);
                                 if assigned(Node) then
                                  Selected:=Node
                               end;
                             end;
                           end;
    end;
end;

function TEnhTreeview.GetCheckBoxes:Boolean;
begin
  result:=fCheckBoxes;
end;

procedure TEnhTreeview.SetCheckBoxes(value:Boolean);
begin
  if fCheckBoxes<>Value then begin
    fCheckBoxes:=Value;
    RecreateWnd;
  end;
end;

function TEnhTreeview.CreateNode: TTreeNode;
begin
  Result := TEnhTreeNode.CreateEnh(Items);
end;

constructor TEnhTreeNode.CreateEnh(AOwner: TTreeNodes);
begin
  inherited Create(AOwner);
end;

destructor TEnhTreeNode.Destroy;
begin
  inherited;
end;

function TEnhTreeNode.GetChecked :Boolean;
var
  Item: TTVItem;
begin
  Result := False;
  with Item do
  begin
    mask := TVIF_STATE;
    hItem := ItemId;
    if TreeView_GetItem(Handle, Item) then
      result:=((Item.State and $2000) = $2000);
  end;
end;

procedure TEnhTreeNode.SetChecked(value:Boolean);
var
  Item: TTVItem;
begin
  if fChecked<>Value then begin
    fChecked:=Value;
    with Item do
    begin
      mask := TVIF_STATE;
      hItem := ItemId;
      if TreeView_GetItem(Handle, Item) then begin
        if fChecked then
          Item.State:=(Item.State or $2000)
        else
          Item.State:=(Item.State or $1000);
        TreeView_SetItem(Handle, Item);
        TreeView.Selected:=self;
      end;
    end;
  end;
end;

procedure Register;
begin
  RegisterComponents('Master', [TEnhTreeView]);
end;

end.
